import { DependencyContainer } from "tsyringe";
import json5 from "json5";

import { IPreSptLoadMod } from "@spt/models/external/IPreSptLoadMod";
import { ILogger } from "@spt/models/spt/utils/ILogger";
import { LogTextColor } from "@spt/models/spt/logging/LogTextColor";
import { IDatabaseTables } from "@spt/models/spt/server/IDatabaseTables";
import { DatabaseServer } from "@spt/servers/DatabaseServer";
import { IPostDBLoadMod } from "@spt/models/external/IPostDBLoadMod";
import { VFS } from "@spt/utils/VFS";

import * as path from "node:path";
import { ConfigServer } from "@spt/servers/ConfigServer";
import { ConfigTypes } from "@spt/models/enums/ConfigTypes";
import { IInsuranceConfig } from "@spt/models/spt/config/IInsuranceConfig";

type ModConfig = {
    PraporMinReturn: number;
    PraporMaxReturn: number;
    PraporMaxStorageTime: number;
    PraporReturnChancePercent: number;

    TherapistMinReturn: number;
    TherapistMaxReturn: number;
    TherapistMaxStorageTime: number;
    TherapistReturnChancePercent: number;
}

class InsuranceTweaks implements IPreSptLoadMod, IPostDBLoadMod
{
    public logger: ILogger;
    public tables: IDatabaseTables;
    public modConfig: ModConfig;
    public configServer: ConfigServer;

    public preSptLoad(container: DependencyContainer): void
    {
        this.logger = container.resolve<ILogger>("WinstonLogger");
        const vfs = container.resolve<VFS>("VFS");
        this.modConfig = json5.parse(vfs.readFile(path.resolve(__dirname, "../config/config.json5")));
    }

    public postDBLoad(container: DependencyContainer): void
    {
        this.tables = container.resolve<DatabaseServer>("DatabaseServer").getTables();
        this.configServer = container.resolve<ConfigServer>("ConfigServer");
        this.traderTweaks(this.modConfig);
    }

    private traderTweaks(modConfig: ModConfig): void
    {
        const traders = this.tables.traders;
        const PRAPOR = "54cb50c76803fa8b248b4571";
        const THERAPIST = "54cb57776803fa99248b456e";
        const insuranceConfig = this.configServer.getConfig<IInsuranceConfig>(ConfigTypes.INSURANCE);

        // Prapor
        traders[PRAPOR].base.insurance.min_return_hour = modConfig.PraporMinReturn;
        traders[PRAPOR].base.insurance.max_return_hour = modConfig.PraporMaxReturn;
        traders[PRAPOR].base.insurance.max_storage_time = modConfig.PraporMaxStorageTime;
        insuranceConfig.returnChancePercent[PRAPOR] = modConfig.PraporReturnChancePercent;
        // Therapist
        traders[THERAPIST].base.insurance.min_return_hour = modConfig.TherapistMinReturn;
        traders[THERAPIST].base.insurance.max_return_hour = modConfig.TherapistMaxReturn;
        traders[THERAPIST].base.insurance.max_storage_time = modConfig.TherapistMaxStorageTime;
        insuranceConfig.returnChancePercent[THERAPIST] = modConfig.TherapistReturnChancePercent;

        this.logger.logWithColor("[InsuranceTweaks] Applied insurance settings", LogTextColor.GREEN);
    }
}

export const mod = new InsuranceTweaks();
